package org.artifactory.ph;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.config.ConfigurationChangesInterceptor;
import org.artifactory.core.api.ph.listener.PackageConfigChangedEventListener;
import org.artifactory.core.api.ph.listener.PackageConfigChangedEventType;
import org.artifactory.descriptor.config.CentralConfigDescriptor;

@Slf4j
public class PackageConfigurationChangesInterceptorImpl implements ConfigurationChangesInterceptor {

    private final PackageConfigChangedEventListener packageConfigChangedEventListener;

    public PackageConfigurationChangesInterceptorImpl(
            PackageConfigChangedEventListener packageConfigChangedEventListener) {
        this.packageConfigChangedEventListener = packageConfigChangedEventListener;
    }

    @Override
    public void onAfterSave(CentralConfigDescriptor newDescriptor, CentralConfigDescriptor oldDescriptor) {
        for (PackageConfigChangedEventType packageConfigChangedEventType : PackageConfigChangedEventType.values()) {
            try {
                PackageConfigChangeType packageConfigChangeType =
                        PackageConfigChangeType.valueOf(packageConfigChangedEventType.name());
                if (packageConfigChangeType.isChanged(newDescriptor, oldDescriptor)) {
                    packageConfigChangedEventListener.onGlobalConfigChange(packageConfigChangedEventType);
                }
            } catch (IllegalArgumentException e) {
                log.debug("Skipping on {} - not found", packageConfigChangedEventType.name());
            }
        }
    }
}
