/*
 *
 * Copyright 2018 JFrog Ltd. All rights reserved.
 * JFROG PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package org.artifactory.repo.service.trash;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.springframework.stereotype.Component;

import java.nio.file.InvalidPathException;
import java.nio.file.Paths;

import static org.artifactory.repo.cache.expirable.InternalMetadataChecker.INTERNAL_METADATA_FOLDER;

/**
 * @author Michael Akushsky
 */
@Slf4j
@Component
public class InternalMetadataNonTrashable implements NonTrashableItem {

    @Override
    public boolean skipTrash(LocalRepoDescriptor descriptor, String path) {
        try {
            return Paths.get(path).startsWith(INTERNAL_METADATA_FOLDER);
        } catch (InvalidPathException e) {
            log.error("Path {} couldn't be parsed.", path);
            log.debug(e.getMessage(), e);
        }
        return false;
    }
}
