package org.artifactory.hqc.cleanup;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.hqc.service.HeavyQueryCacheService;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Spy;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.verify;
import static org.mockito.MockitoAnnotations.openMocks;

/**
 * @author Andrey Tashlitsky
 */
class HeavyQueryCacheCleanupJobTest {

    @Mock
    private ArtifactoryContext artifactoryContext;
    @Mock
    private HeavyQueryCacheService heavyQueryCacheService;
    @Mock
    private JobExecutionContext jobExecutionContext;
    @Spy
    private HeavyQueryCacheCleanupJob heavyQueryCacheCleanupJob;
    private AutoCloseable autoCloseable;

    @BeforeEach
    public void beforeEach() {
        autoCloseable = openMocks(this);
        doReturn(heavyQueryCacheService).when(artifactoryContext).beanForType(HeavyQueryCacheService.class);
        ArtifactoryContextThreadBinder.bind(artifactoryContext);
    }

    @AfterEach
    public void afterEach() throws Exception {
        ArtifactoryContextThreadBinder.unbind();
        autoCloseable.close();
    }

    @Test
    void heavyQueryCacheCleanupJobTest() throws JobExecutionException {
        heavyQueryCacheCleanupJob.onExecute(jobExecutionContext);
        verify(heavyQueryCacheService).runCleanup();
    }
}
