package org.artifactory.ph;

import org.artifactory.core.api.ph.listener.PackageConfigChangedEventListener;
import org.artifactory.core.api.ph.listener.PackageConfigChangedEventType;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;

class PackageConfigurationChangesInterceptorImplTest {

    @Mock
    PackageConfigChangedEventListener packageConfigChangedEventListener;
    @Mock
    CentralConfigDescriptor oldDescriptor;
    @Mock
    CentralConfigDescriptor newDescriptor;
    PackageConfigurationChangesInterceptorImpl packageConfigurationChangesInterceptor;

    @BeforeEach
    void setup() {
        MockitoAnnotations.openMocks(this);
        packageConfigurationChangesInterceptor =
                spy(new PackageConfigurationChangesInterceptorImpl(packageConfigChangedEventListener));
    }

    @AfterEach
    void cleanup() {
        verifyNoMoreInteractions(packageConfigChangedEventListener);
    }


    @Test
    void testAfterSaveWithoutChanges() {
        packageConfigurationChangesInterceptor.onAfterSave(newDescriptor, null);
        verify(this.packageConfigChangedEventListener, never()).onGlobalConfigChange(any());
    }

    @Test
    void testAfterSave() {
        when(oldDescriptor.getUrlBase()).thenReturn("http://localhost:8082");
        PackageConfigChangedEventListener packageConfigChangedEventListener =
                Mockito.mock(PackageConfigChangedEventListener.class);
        doAnswer(invocationOnMock -> {
            Assertions.assertEquals(PackageConfigChangedEventType.BASE_URL, invocationOnMock.getArgument(0));
            return null;
        }).when(packageConfigChangedEventListener).onGlobalConfigChange(any());

        packageConfigurationChangesInterceptor.onAfterSave(newDescriptor, oldDescriptor);
        verify(this.packageConfigChangedEventListener, times(1)).onGlobalConfigChange(any());
        verifyNoMoreInteractions(packageConfigChangedEventListener);
    }
}