package org.artifactory.repo.system.suppliers;

import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.testng.annotations.Test;

import static org.artifactory.repo.config.RepoConfigDefaultValues.DEFAULT_REPO_LAYOUT;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

public class BuildInfoRepoTest {

    @Test
    public void testBuildInfoBuildDescriptor() {
        CentralConfigService config = mock(CentralConfigService.class);
        CentralConfigDescriptor configDescriptor = mock(CentralConfigDescriptor.class);
        BuildInfoRepo buildInfoRepo = new BuildInfoRepo(config);
        when(config.getDescriptor()).thenReturn(configDescriptor);

        LocalRepoDescriptor localRepoDescriptor = buildInfoRepo.buildDescriptor();
        assertEquals(localRepoDescriptor.getType(), RepoType.BuildInfo);
        verify(configDescriptor, times(1)).getRepoLayout(DEFAULT_REPO_LAYOUT);
    }

    @Test
    public void testBuildInfoBuildDescriptorWithRepoName() {
        CentralConfigService config = mock(CentralConfigService.class);
        CentralConfigDescriptor configDescriptor = mock(CentralConfigDescriptor.class);
        BuildInfoRepo buildInfoRepo = new BuildInfoRepo(config);
        when(config.getDescriptor()).thenReturn(configDescriptor);

        LocalRepoDescriptor localRepoDescriptor = buildInfoRepo.buildDescriptor("repo");
        assertEquals(localRepoDescriptor.getType(), RepoType.BuildInfo);
        assertEquals(localRepoDescriptor.getKey(), "repo");
        verify(configDescriptor, times(1)).getRepoLayout(DEFAULT_REPO_LAYOUT);
    }
}