package org.artifactory.repo;

import com.fasterxml.jackson.annotation.JsonFilter;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.artifactory.descriptor.repo.FederatedMember;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;

import java.util.List;
import java.util.stream.Collectors;

import static org.artifactory.repo.RepoDetailsType.FEDERATED;

/**
 * Federated repository configuration
 *
 * @author Gidi Shabat
 * @author Andrei Komarov
 * @see FederatedRepoDescriptor
 */
@Data
@EqualsAndHashCode(callSuper = false)
@JsonFilter("typeSpecificFilter")
@NoArgsConstructor
public class FederatedRepositoryConfigurationImpl extends LocalRepositoryConfigurationImpl
        implements FederatedRepositoryConfiguration {

    public static final String TYPE = "federated";
    @IncludeTypeSpecific(repoType = FEDERATED)
    private List<FederatedMemberConfiguration> members;
    private String proxy;
    private boolean disableProxy;

    public FederatedRepositoryConfigurationImpl(FederatedRepoDescriptor federatedRepoDescriptor) {
        super(federatedRepoDescriptor, TYPE);
        List<FederatedMember> descriptorMembers = federatedRepoDescriptor.getFederatedMembers();
        if (descriptorMembers != null && !descriptorMembers.isEmpty()) {
            setMembers(descriptorMembers.stream().map(this::toFederatedMemberConfig).collect(Collectors.toList()));
        } else {
            setPropertySets(Lists.newArrayList());
        }
        if (!federatedRepoDescriptor.isDisableProxy() && federatedRepoDescriptor.getProxy() != null) {
            proxy = federatedRepoDescriptor.getProxy().getKey();
        }
        disableProxy = federatedRepoDescriptor.isDisableProxy();
    }

    private FederatedMemberConfiguration toFederatedMemberConfig(FederatedMember a) {
        return new FederatedMemberConfiguration(a.getUrl(), a.getMode(), a.isEnabled());
    }

    @Override
    public String getProxy() {
        return proxy;
    }

    public void setProxy(String proxy) {
        this.proxy = proxy;
    }

    @Override
    public boolean isDisableProxy() {
        return disableProxy;
    }

    public void setDisabledProxy(boolean disableProxy) {
        this.disableProxy = disableProxy;
    }
}
