package org.artifactory.version.converter.v3134;

import org.artifactory.config.AccessConfigHolder;
import org.artifactory.convert.XmlConverterTest;
import org.junit.jupiter.api.Test;

import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

class CopyPasswordExpirationPolicyToConfigServiceConverterTest extends XmlConverterTest {
    private static final String RESOURCE_PATH =
            "/config/test/config.3.1.32_removeAccessConfigAndAnonAccessEnabledAndUserLockPolicy.xml";
    private static final String RESOURCE_PATH_FALSE =
            "/config/test/config.3.1.32_removeAccessConfigAndAnonAccessEnabledAndUserLockPolicyFalse.xml";
    private static final String RESOURCE_PATH_NOT_EXISTS =
            "/config/test/config.3.1.32_removeAccessConfigAndAnonAccessEnabledAndUserLockPolicyNotExists.xml";

    @Test
    void testConvert() throws Exception {
        convertXml(RESOURCE_PATH,
                new CopyPasswordExpirationPolicyToConfigServiceConverter());
        Map<String, Object> actual = AccessConfigHolder.getInstance().drainConfig();
        Map<String, Object> expected = Map.of("security",
                Map.of("user-lock-policy", Map.of("password-expiry-days", 60)));
        assertEquals(expected, actual);
    }

    @Test
    void testConvertFalse() throws Exception {
        convertXml(RESOURCE_PATH_FALSE,
                new CopyPasswordExpirationPolicyToConfigServiceConverter());
        Map<String, Object> actual = AccessConfigHolder.getInstance().drainConfig();
        Map<String, Object> expected = Map.of("security",
                Map.of("user-lock-policy", Map.of("password-expiry-days", 0)));
        assertEquals(expected, actual);
    }

    @Test
    void testConvertNotExists() throws Exception {
        convertXml(RESOURCE_PATH_NOT_EXISTS,
                new CopyPasswordExpirationPolicyToConfigServiceConverter());
        Map<String, Object> actual = AccessConfigHolder.getInstance().drainConfig();
        assertTrue(actual.isEmpty());
    }
}
