package org.artifactory.core.api.ph.exception;

import java.util.Map;

public class PackageException extends Exception {
    private final int status;
    private final String publicReason;
    private final String internalReason;
    private final Map<String, String> additionalHeaders;

    public PackageException(int status, String publicReason, String internalReason) {
        super(publicReason);
        this.status = status;
        this.publicReason = publicReason;
        this.internalReason = internalReason;
        this.additionalHeaders = Map.of();
    }

    public PackageException(int status, String publicReason, String internalReason,
            Map<String, String> additionalHeaders) {
        super(publicReason);
        this.status = status;
        this.publicReason = publicReason;
        this.internalReason = internalReason;
        this.additionalHeaders = additionalHeaders;
    }

    public PackageException(int status, String publicReason, String internalReason, Exception cause) {
        super(publicReason, cause);
        this.status = status;
        this.publicReason = publicReason;
        this.internalReason = internalReason;
        this.additionalHeaders = Map.of();
    }

    public int getStatus() {
        return status;
    }

    public String getPublicReason() {
        return publicReason;
    }

    public String getInternalReason() {
        return internalReason;
    }

    public Map<String, String> getAdditionalHeaders() {
        return additionalHeaders;
    }
}
