package org.artifactory.storage.db.aql.itest.service.optimizer;

import org.artifactory.aql.model.AqlComparatorEnum;
import org.artifactory.aql.result.rows.AqlRowResult;
import org.artifactory.storage.db.aql.sql.builder.query.aql.AqlQuery;
import org.artifactory.storage.db.aql.sql.builder.query.aql.ComplexPropertyCriterion;
import org.artifactory.storage.db.aql.sql.builder.query.aql.ComplexPropertyCriterionExists;
import org.artifactory.storage.db.aql.sql.builder.query.aql.Criterion;
import org.artifactory.storage.db.aql.sql.builder.query.aql.SimplePropertyCriterion;
import org.artifactory.storage.db.aql.sql.builder.query.aql.SimplePropertyCriterionExists;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class FromLeftOuterJoinToExists extends OptimizationStrategy {

    private static final Pattern pattern = Pattern.compile("[p,k,v]");

    @Override
    public <T extends AqlRowResult> void optimize(AqlQuery<T> aqlQuery, String transformation) {
        if (shouldSkipOptimization(aqlQuery)) {
            return;
        }
        // Try to find matching sub strings
        Matcher matcher = pattern.matcher(transformation);
        // Each match represent sub query that can be optimize
        while (matcher.find()) {
            // Get sub query bounds
            int start = matcher.start();
            int end = matcher.end();

            for (int i = start; i < end; i++) {
                if (transformation.charAt(i) == 'p') {
                    ComplexPropertyCriterion criteria = (ComplexPropertyCriterion) aqlQuery.getAqlElements().get(i);
                    AqlComparatorEnum comparator = AqlComparatorEnum.value(criteria.getComparatorName());
                    if (!(criteria.isMspOperator() || criteria.getTable1().getId() == -1)) {
                        switch (comparator) {
                            case matches:
                            case equals:
                            case notMatches:
                            case notEquals: {
                                Criterion newCriteria =
                                        new ComplexPropertyCriterionExists(criteria.getSubDomains(),
                                                criteria.getVariable1(), criteria.getTable1(),
                                                criteria.getComparatorName(),
                                                criteria.getVariable2(),
                                                criteria.getTable2(), criteria.isMspOperator());
                                aqlQuery.getAqlElements().remove(i);
                                aqlQuery.getAqlElements().add(i, newCriteria);
                            }
                            default:
                        }
                    }
                }
                if (transformation.charAt(i) == 'k' || transformation.charAt(i) == 'v') {
                    Criterion criterion = (Criterion) aqlQuery.getAqlElements().get(i);
                    if (!criterion.isMspOperator()) {
                        if (criterion instanceof SimplePropertyCriterion) {
                            AqlComparatorEnum comparator = AqlComparatorEnum.value(criterion.getComparatorName());
                            switch (comparator) {
                                case matches:
                                case equals:
                                case notMatches:
                                case notEquals: {
                                    Criterion newCriteria =
                                            new SimplePropertyCriterionExists(criterion.getSubDomains(),
                                                    criterion.getVariable1(), criterion.getTable1(),
                                                    criterion.getComparatorName(),
                                                    criterion.getVariable2(),
                                                    criterion.getTable2(), criterion.isMspOperator());
                                    aqlQuery.getAqlElements().remove(i);
                                    aqlQuery.getAqlElements().add(i, newCriteria);
                                }
                                default:
                            }
                        }
                    }
                }
            }
        }
    }
}
