package org.artifactory.storage.db.aql.itest.service.decorator;


import org.artifactory.aql.model.AqlComparatorEnum;
import org.artifactory.aql.result.rows.AqlRowResult;
import org.artifactory.storage.db.aql.sql.builder.query.aql.AqlQuery;
import org.artifactory.storage.db.aql.sql.builder.query.aql.Criterion;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

import java.util.ArrayList;
import java.util.stream.Stream;

import static org.testng.Assert.assertEquals;

/**
 * @author Tomer Slutsky
 */
public class SpecialReposDecoratorTest extends BaseDecorator {
    private static Stream<Arguments> provide() {
        return Stream.of(
                Arguments.of(new TrashcanDecorator(), "( item.repo $eq repo1 ) $and item.repo $ne auto-trashcan"),
                Arguments.of(new SupportBundlesRepoDecorator(),
                        "( item.repo $eq repo1 ) $and item.repo $ne jfrog-support-bundle"));
    }

    @ParameterizedTest
    @MethodSource("provide")
    public void testDecorateItemsSearch(SpecialReposDecorator decorator, String expected) {
        AqlQuery<AqlRowResult> aqlQuery = new AqlQuery<>();
        Criterion originalCriterion = createRepoCriteria(AqlComparatorEnum.equals, "repo1");
        aqlQuery.getAqlElements().add(originalCriterion);
        aqlQuery.putSubQuery(new ArrayList<>(aqlQuery.getAqlElements()));
        decorator.decorateItemsSearch(aqlQuery);
        assertEquals(toString(aqlQuery.getAqlElements()), expected);
        assertEquals(toString(aqlQuery.getUnionSubQueries().get(0).getAqlElements()), expected);
    }
}