package org.artifactory.storage.db.aql.itest.service.optimizer;


import org.artifactory.aql.model.AqlFunctionFieldEnum;
import org.artifactory.aql.model.AqlFunctionTableEnum;
import org.artifactory.aql.model.AqlSqlFunction;
import org.artifactory.storage.db.aql.sql.builder.query.aql.AqlQuery;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * @author Tomer Slutsky
 */
class OptimizationStrategyTest {
    private static Stream<Arguments> provide() {
        return Stream.of(
                Arguments.of(
                        new AqlSqlFunction(AqlFunctionFieldEnum.COUNT, AqlFunctionTableEnum.ARTIFACT_NODE_ID, false),
                        false, false, true),
                Arguments.of(
                        new AqlSqlFunction(AqlFunctionFieldEnum.COUNT, AqlFunctionTableEnum.ARTIFACT_NODE_ID, false),
                        false, true, true),
                Arguments.of(
                        new AqlSqlFunction(AqlFunctionFieldEnum.COUNT, AqlFunctionTableEnum.ARTIFACT_NODE_ID, true),
                        true, false, false),
                Arguments.of(
                        new AqlSqlFunction(AqlFunctionFieldEnum.COUNT, AqlFunctionTableEnum.ARTIFACT_NODE_ID, true),
                        true, true, true),
                Arguments.of(null, false, true, true),
                Arguments.of(null, false, false, false),
                Arguments.of(null, true, true, true),
                Arguments.of(null, true, false, false)
        );
    }

    @ParameterizedTest
    @MethodSource("provide")
    void testSkipOptimization(AqlSqlFunction function, boolean distinct, boolean isNullExists,
            boolean expected) {
        AqlQuery query = new AqlQuery();
        query.setSqlFunction(function);
        query.setDistinct(distinct);
        query.setNullExists(isNullExists);
        assertEquals(OptimizationStrategy.shouldSkipOptimization(query), expected);
    }
}