package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.jfrog.converter.model.ChangeLogContext;
import org.jfrog.storage.util.DbUtils;
import org.testng.annotations.Test;

import static org.jfrog.storage.util.DbUtils.withMetadata;
import static org.testng.Assert.assertTrue;

public class V95UpgradeNodeEventsTmpEventIdIdxTest extends UpgradeBaseTest {

    @Test
    public void test() throws Exception {

        //Drop the index if exist
        if (indexExist()) {
            String indexName = withMetadata(jdbcHelper,
                    metadata -> DbUtils.normalizedName("node_events_tmp_event_id_idx", metadata));
            jdbcHelper.executeUpdate("DROP INDEX " + indexName);
        }

        //Run converters, index should be created.
        final ArtifactoryDBVersion dbVersion = ArtifactoryDBVersion.v95;
        try (ChangeLogContext context = new ChangeLogContext(jdbcHelper.getDataSource().getConnection(),
                dbProperties.getDbType(), dbVersion)) {
            dbVersion.converter().convert(context);
        }

        //Verify index created
        assertTrue(indexExist());

        //Run converters again, now it should not convert
        try (ChangeLogContext context = new ChangeLogContext(jdbcHelper.getDataSource().getConnection(),
                dbProperties.getDbType(), dbVersion)) {
            dbVersion.converter().convert(context);
        }
    }

    private Boolean indexExist() throws java.sql.SQLException {
        return DbUtils.withConnection(jdbcHelper,
                conn -> DbUtils
                        .indexExists(conn, dbProperties.getDbType(), "node_events_tmp", "tmp_event_id",
                                "node_events_tmp_event_id_idx", true));
    }

    @Override
    protected ArtifactoryDBVersion upToVersion() {
        return ArtifactoryDBVersion.v95;
    }
}
