package org.artifactory.ui.rest.service.admin.configuration.repositories.util;

import org.artifactory.build.InternalBuildService;
import org.artifactory.federation.FederatedRepoService;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedBuildInfoDetails;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import static org.apache.http.HttpStatus.SC_OK;

@Component
@Scope(BeanDefinition.SCOPE_SINGLETON)
public class GetFederatedBuildInfoDetailsService implements RestService<String> {

    @Autowired
    private InternalBuildService buildService;

    @Autowired
    private FederatedRepoService federatedRepoService;

    @Override
    public void execute(ArtifactoryRestRequest<String> request, RestResponse response) {
        String projectKey = request.getImodel();
        FederatedBuildInfoDetails federatedBuildInfoDetails = new FederatedBuildInfoDetails();
        setBuildInfoDetailsToModel(
                federatedRepoService.isProjectBuildInfoRepoFederated(projectKey),
                buildService.getBuildInfoRepoKeyByProject(projectKey),
                federatedBuildInfoDetails
        );
        response.responseCode(SC_OK).iModel(federatedBuildInfoDetails);
    }

    private void setBuildInfoDetailsToModel(boolean isFederatedRepo, String repoKey,
            FederatedBuildInfoDetails buildInfoDetails) {
        buildInfoDetails.setBuildInfoRepo(repoKey);
        buildInfoDetails.setEnableConvert(!isFederatedRepo);
    }
}
