/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.admin.security.saml;

import org.apache.commons.lang.StringUtils;
import org.artifactory.addon.sso.saml.SamlException;
import org.artifactory.addon.sso.saml.SamlHandler;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.rest.common.util.RequestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @author Gidi Shabat
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class GetSamlLoginResponseService implements RestService {
    private static final Logger log = LoggerFactory.getLogger(GetSamlLoginResponseService.class);

    @Autowired
    private SamlHandler samlHandler;

    @Override
    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        try {
            Map<String, List<String>> formParameters = RequestUtils.getFormParameters(request);
            samlHandler.handleLoginResponse(request.getServletRequest(), response.getServletResponse(), formParameters);
        } catch (SamlException e) {
            log.error("Error occurred while trying to login using SAML: {}", e.getMessage());
            log.debug("Error occurred while trying to login using SAML", e);
            if (StringUtils.isNotEmpty(e.getUiErrorMessage())) {
                response.error(String.format(
                        "Error occurred while trying to login using SAML: %s. Check your Artifactory logs for more " +
                                "details.",
                        e.getUiErrorMessage()));
            } else {
                response.error(
                        "Error occurred while trying to login using SAML. Check your Artifactory logs for more " +
                                "details.");
            }
        } catch (Exception e) {
            log.error("Error occurred while trying to login using SAML: {}", e.getMessage());
            log.debug("Error occurred while trying to login using SAML", e);
            response.error(
                    "Error occurred while trying to login using SAML. Check your Artifactory logs for more details.");
        }
    }
}
