package org.artifactory.ui.rest.service.admin.configuration.general;

import org.apache.commons.lang3.StringUtils;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.service.ConfigValidationService;
import org.artifactory.ui.rest.model.admin.configuration.generalconfig.GeneralConfig;
import org.jetbrains.annotations.NotNull;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import static org.artifactory.ui.rest.service.admin.configuration.general.UpdateConfigUtils.prepareAddonsManager;
import static org.artifactory.ui.rest.service.admin.configuration.general.UpdateConfigUtils.prepareMutableCentralConfigDescriptor;

/**
 * @author Alexei Vainshtein
 */
public class UpdateGeneralConfigServiceTest {

    private UpdateGeneralConfigService updateGeneralConfigService;

    @BeforeClass
    public void beforeClass() {
        MutableCentralConfigDescriptor descriptor = prepareMutableCentralConfigDescriptor();
        CentralConfigService testCentralConfigService = Mockito.mock(CentralConfigService.class);
        Mockito.when(testCentralConfigService.getMutableDescriptor()).thenReturn(descriptor);
        updateGeneralConfigService = new UpdateGeneralConfigService(testCentralConfigService, prepareAddonsManager(),
                new ConfigValidationService());
        Mockito.doAnswer(invocationOnMock -> {
            MutableCentralConfigDescriptor mutableCentralConfigDescriptor =
                    (MutableCentralConfigDescriptor) invocationOnMock.getArguments()[0];
            Assert.assertTrue(mutableCentralConfigDescriptor.isArchiveIndexEnabled());
            return null;
        }).when(testCentralConfigService).saveEditedDescriptorAndReload(descriptor);
    }

    @Test
    public void testUpdateDescriptorAndSave() {
        updateGeneralConfigService.updateDescriptorAndSave(prepareGeneralConfig("cool"));
    }

    @Test
    public void testUpdateDescriptorAndSaveWithLogoUrl() {
        updateGeneralConfigService.updateDescriptorAndSave(
                prepareGeneralConfig("cool", "https://artifactory.com/logo.png"));
    }

    @Test(expectedExceptions = IllegalArgumentException.class,
            expectedExceptionsMessageRegExp = "Logo URL is not valid")
    public void testUpdateDescriptorAndSaveBadLogoUrl() {
        updateGeneralConfigService.updateDescriptorAndSave(
                prepareGeneralConfig("cool", "https://artifactory.com/1.php"));
    }

    @Test(expectedExceptions = IllegalArgumentException.class,
            expectedExceptionsMessageRegExp = "Illegal name.*is not allowed")
    public void testUpdateDescriptorAndSaveBadServerName() {
        updateGeneralConfigService.updateDescriptorAndSave(prepareGeneralConfig("<a href=http://attacker.com a"));
    }

    @Test
    public void testUpdateDescriptorAndSaveBlankServerName() {
        updateGeneralConfigService.updateDescriptorAndSave(prepareGeneralConfig(StringUtils.EMPTY));
    }

    @NotNull
    public GeneralConfig prepareGeneralConfig(String serverName) {
        return prepareGeneralConfig(serverName, null);
    }

    @NotNull
    public GeneralConfig prepareGeneralConfig(String serverName, String logoUrl) {
        GeneralConfig config = new GeneralConfig();
        config.setServerName(serverName);
        config.setLogoUrl(logoUrl);
        config.setArchiveIndexEnabled(true);
        config.setHelpLinksEnabled(true);
        config.setFileUploadMaxSize(10);
        config.setShowAddonSettings(false);
        config.setGlobalOfflineMode(true);
        config.setSystemMessageEnabled(false);
        config.setShowSystemMessageOnAllPages(false);
        config.setFolderDownloadEnabled(false);
        config.setFolderDownloadEnabled(false);
        return config;
    }


}