package org.artifactory.ui.rest.service.admin.configuration.repositories.util;


import org.artifactory.addon.AddonsManager;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.CentralConfigDescriptorImpl;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.projects.ProjectsService;
import org.artifactory.rest.common.exception.RepoConfigException;
import org.artifactory.service.ConfigValidationService;
import org.artifactory.ui.rest.model.admin.configuration.repository.GeneralRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.HelmTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualSelectedRepository;
import org.artifactory.ui.rest.service.admin.configuration.repositories.replication.ReplicationConfigService;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder.ReplicationConfigDescriptorBuilder;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder.RepoConfigDescriptorBuilder;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.validator.ReplicationConfigValidator;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.validator.RepoConfigValidator;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import java.util.List;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.when;

class CreateRepoConfigHelperTest {

    @Mock
    private AddonsManager addonsManager;
    @Mock
    private ReplicationConfigDescriptorBuilder replicationBuilder;
    @Mock
    private ReplicationConfigService replicationConfigService;
    @Mock
    private ReplicationConfigValidator replicationValidator;
    @Mock
    private RepoConfigValidator repoValidator;
    @Mock
    private ConfigValidationService configValidationService;
    @Mock
    private ProjectsService projectsService;
    @Mock
    private CentralConfigService configService;

    private AutoCloseable mocks;

    CreateRepoConfigHelper helper;

    @BeforeEach
    public void setup() {
        mocks = MockitoAnnotations.openMocks(this);
        helper = new CreateRepoConfigHelper();
        helper.setAddonsManager(addonsManager);
        helper.setReplicationBuilder(replicationBuilder);
        helper.setReplicationConfigService(replicationConfigService);
        helper.setReplicationValidator(replicationValidator);
        helper.setRepoValidator(repoValidator);
        helper.setConfigValidationService(configValidationService);
        helper.setProjectsService(projectsService);

        RepoConfigDescriptorBuilder repoBuilder = new RepoConfigDescriptorBuilder();
        repoBuilder.setConfigValidator(repoValidator);
        repoBuilder.setCentralConfig(configService);
        helper.setRepoBuilder(repoBuilder);

        when(configService.getMutableDescriptor()).thenReturn(new CentralConfigDescriptorImpl());
    }

    @AfterEach
    public void tearDown() throws Exception {
        mocks.close();
    }

    @ParameterizedTest
    @ValueSource(booleans = {true, false})
    void testHandleVirtual(boolean included) throws RepoConfigException {
        VirtualRepositoryConfigModel configModel = new VirtualRepositoryConfigModel();
        VirtualBasicRepositoryConfigModel basic = new VirtualBasicRepositoryConfigModel();
        LocalRepoDescriptor descriptor = new LocalRepoDescriptor();
        descriptor.setKey("local-repo");
        basic.setSelectedRepositories(List.of(new VirtualSelectedRepository(descriptor)));
        configModel.setBasic(basic);
        GeneralRepositoryConfigModel general = new GeneralRepositoryConfigModel();
        general.setRepoKey("repo");
        general.setProjectKey("project");
        general.setEnvironments(Set.of("PROD"));
        configModel.setGeneral(general);
        configModel.setAdvanced(new VirtualAdvancedRepositoryConfigModel());
        configModel.setTypeSpecific(new HelmTypeSpecificConfigModel());
        when(projectsService.getProjectKeyOfRepository("repo")).thenReturn("project");
        CentralConfigDescriptorImpl configDescriptor = new CentralConfigDescriptorImpl();
        if (included) {
            when(projectsService.getProjectRepositoriesKeys("project", false)).thenReturn(Set.of("local-repo"));
            helper.handleVirtual(configModel, configDescriptor);
        } else {
            IllegalArgumentException illegalArgumentException = assertThrows(IllegalArgumentException.class,
                    () -> helper.handleVirtual(configModel, configDescriptor));
            assertEquals("Virtual repository must include only repositories from the same project it was created with",
                    illegalArgumentException.getMessage());
        }
    }
}