package org.artifactory.ui.utils;

import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.fs.ItemInfo;
import org.artifactory.model.xstream.fs.FileInfoImpl;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.junit.jupiter.api.Assertions;
import org.testng.annotations.Test;

import java.util.List;

class BrowseUtilsTest extends ArtifactoryHomeBoundTest {

    @Test
    void testFilterItemsIfNeeded() {
        List<ItemInfo> items = List.of(
                new FileInfoImpl(RepoPathFactory.create("repo", "")),
                new FileInfoImpl(RepoPathFactory.create("repo", ".git")),
                new FileInfoImpl(RepoPathFactory.create("repo", ".git/yo")),
                new FileInfoImpl(RepoPathFactory.create("repo", ".got/yo")),
                new FileInfoImpl(RepoPathFactory.create("repo", "git/yo")),
                new FileInfoImpl(RepoPathFactory.create("repo", "git")),
                new FileInfoImpl(RepoPathFactory.create("repo", "a.sha1")),
                new FileInfoImpl(RepoPathFactory.create("repo", "a.sha256")),
                new FileInfoImpl(RepoPathFactory.create("repo", "a.md5"))
        );

        List<ItemInfo> nonCargoResults = BrowseUtils.filterItemsIfNeeded(items, null);
        Assertions.assertEquals(6, nonCargoResults.size());

        List<ItemInfo> cargoResults = BrowseUtils.filterItemsIfNeeded(items, RepoType.Cargo);
        Assertions.assertEquals(4, cargoResults.size());
        cargoResults.forEach(item -> Assertions.assertFalse(item.getRelPath().startsWith(".git")));
    }

    @Test
    void testFilterInternalMetadata() {
        List<ItemInfo> items = List.of(
                new FileInfoImpl(RepoPathFactory.create("repo", "")),
                new FileInfoImpl(RepoPathFactory.create("repo", ".jfrog/repository.catalog")),
                new FileInfoImpl(RepoPathFactory.create("repo",
                        "sha256:6e2b7e3d21e385681a38a8d66a40213462762e7e14c6e97b7701cee8b21e32d6")),
                new FileInfoImpl(RepoPathFactory.create("repo",
                        "sha256:93361007a78c4b38d0c868599685aa8a08fb3c27067933243c719cb71b514184")),
                new FileInfoImpl(RepoPathFactory.create("repo", "manifest.json")),
                new FileInfoImpl(RepoPathFactory.create("repo", "a.sha1")),
                new FileInfoImpl(RepoPathFactory.create("repo", "a.sha256")),
                new FileInfoImpl(RepoPathFactory.create("repo", "a.md5"))
        );

        List<ItemInfo> dockerResults = BrowseUtils.filterItemsIfNeeded(items, null);
        Assertions.assertEquals(4, dockerResults.size());
    }

}