package org.artifactory.rest.services.retention;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.retention.ArtifactRetentionAddon;
import org.artifactory.addon.retention.ArtifactRetentionPolicyBaseModel;
import org.artifactory.addon.retention.PolicyTranslator;
import org.artifactory.descriptor.retention.ArtifactRetentionPolicyDescriptor;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import static org.artifactory.descriptor.retention.ArtifactRetentionOperationType.ARCHIVE;

/**
 * @author Omri Ziv
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class UpdateArtifactRetentionPolicyService implements RestService<ArtifactRetentionPolicyBaseModel> {

    @Autowired
    private AddonsManager addonsManager;

    @Override
    public void execute(ArtifactoryRestRequest<ArtifactRetentionPolicyBaseModel> request, RestResponse response) {
        ArtifactRetentionPolicyBaseModel policyModelBase = request.getImodel();
        String policyKey = policyModelBase.getKey();
        ArtifactRetentionPolicyDescriptor
                policyDescriptor = PolicyTranslator.toPolicyDescriptor(policyModelBase, ARCHIVE);
        ArtifactRetentionPolicyBaseModel policyRetrieved = addonsManager.addonByType(ArtifactRetentionAddon.class)
                .updatePolicy(policyKey, policyDescriptor);
        response.iModel(policyRetrieved);
    }
}
